]> git.cworth.org Git - empires-server/blobdiff - game.js
test: Maintain a list of activated players for automated cleanup
[empires-server] / game.js
diff --git a/game.js b/game.js
index 34a0d13b4168868a4ed6e55d275c2139b0774dbc..7ca150735269d0c0b6c4bc0681e70302d4a41813 100644 (file)
--- a/game.js
+++ b/game.js
@@ -1,3 +1,5 @@
+const no_team = { name: "" };
+
 /* A single player can have multiple connections, (think, multiple
  * browser windows with a common session cookie). */
 class Player {
@@ -6,7 +8,8 @@ class Player {
     this.session_id = session_id;
     this.name = name;
     this.connections = [connection];
-    this.team = "";
+    this.team = no_team;
+    this.active = true;
   }
 
   add_connection(connection) {
@@ -21,7 +24,7 @@ class Player {
 
   /* Returns the number of remaining connections after this one is removed. */
   remove_connection(connection) {
-    this.connections.filter(c => c !== connection);
+    this.connections = this.connections.filter(c => c !== connection);
     return this.connections.length;
   }
 
@@ -34,11 +37,22 @@ class Player {
     return JSON.stringify({
       id: this.id,
       name: this.name,
-      team: this.team
+      team: this.team.name,
+      score: this.score
     });
   }
 }
 
+/* This replacer function allows for JSON.stringify to give results
+ * for objects of various types that are used in game state.
+ */
+function stringify_replacer(key, value) {
+  if (typeof value === 'object' && value instanceof Set) {
+    return [...value];
+  }
+  return value;
+}
+
 /* Base class providing common code for game engine implementations. */
 class Game {
   constructor(id) {
@@ -48,7 +62,7 @@ class Game {
     this.next_player_id = 1;
     this.teams = [];
     this.state = {
-      team_to_play: ""
+      team_to_play: no_team
     };
     this.first_move = true;
 
@@ -94,7 +108,7 @@ class Game {
       }
 
       /* Cannot move if you are not on a team. */
-      if (player.team === "")
+      if (player.team === no_team)
       {
         return { legal: false,
                  message: "You must be on a team to take a turn" };
@@ -115,7 +129,7 @@ class Game {
    * Return true if assignment made, false otherwise. */
   assign_player_to_team_perhaps(player, team)
   {
-    if (player.team !== "")
+    if (player.team !== no_team)
       return false;
 
     player.team = team;
@@ -138,7 +152,7 @@ class Game {
 
       /* Yes, start at 1 to skip teams[0] which we just assigned. */
       for (let i = 1; i < this.teams.length; i++) {
-        const other = this.players.find(p => p !== player && p.team === "");
+        const other = this.players.find(p => (p !== player) && (p.team === no_team));
         if (!other)
           return;
         this.assign_player_to_team_perhaps(other, this.teams[i]);
@@ -156,7 +170,20 @@ class Game {
 
     /* No existing player. Add a new one. */
     const id = this.next_player_id;
-    const player = new Player(id, session.id, session.nickname, connection);
+    let nickname = session.nickname;
+    if (nickname === "")
+      nickname = "Guest";
+    const nickname_orig = nickname;
+
+    /* Ensure we don't have a name collision with a previous player. */
+    let unique_suffix = 1;
+    while (this.players.find(player => player.name === nickname))
+    {
+      nickname = `${nickname_orig}${unique_suffix.toString().padStart(2, '0')}`;
+      unique_suffix++;
+    }
+
+    const player = new Player(id, session.id, nickname, connection);
 
     /* Broadcast before adding player to list (to avoid announcing the
      * new player to itself). */
@@ -167,6 +194,19 @@ class Game {
     this.players_by_session[session.id] = player;
     this.next_player_id++;
 
+    /* After adding the player to the list, and if we are already past
+     * the first move, assign this player to the first team that
+     * doesn't already have a player assigned (if any). */
+    if (! this.first_move) {
+      const have_players = Array(this.teams.length).fill(false);
+      this.players.forEach(p => {
+        if (p.team.id !== undefined)
+          have_players[p.team.id] = true;
+      });
+      const first_empty = have_players.findIndex(i => i === false);
+      this.assign_player_to_team_perhaps(player, this.teams[first_empty]);
+    }
+
     return player;
   }
 
@@ -175,9 +215,8 @@ class Game {
   remove_player_connection(player, connection) {
     const remaining = player.remove_connection(connection);
     if (remaining === 0) {
+      player.active = false;
       const player_data = JSON.stringify({ id: player.id });
-      this.players.filter(p => p !== player);
-      delete this.players_by_session[player.session_id];
       this.broadcast_event("player-exit", player_data);
     }
   }
@@ -196,6 +235,10 @@ class Game {
     this.broadcast_string(`event: ${type}\ndata: ${data}\n`);
   }
 
+  broadcast_event_object(type, obj) {
+    this.broadcast_event(type, JSON.stringify(obj));
+  }
+
   handle_events(request, response) {
     /* These headers will keep the connection open so we can stream events. */
     const headers = {
@@ -223,15 +266,15 @@ class Game {
     /* And the player-info event. */
     response.write(`event: player-info\ndata: ${player.info_json()}\n\n`);
 
-    /* As well as player-enter events for all existing players. */
-    this.players.filter(p => p !== player).forEach(p => {
+    /* As well as player-enter events for all existing, active players. */
+    this.players.filter(p => p !== player && p.active).forEach(p => {
       response.write(`event: player-enter\ndata: ${p.info_json()}\n\n`);
     });
 
     /* Finally, if this game class has a "state" property, stream that
      * current state to the client. */
     if (this.state) {
-      const state_json = JSON.stringify(this.state);
+      const state_json = JSON.stringify(this.state, stringify_replacer);
       response.write(`event: game-state\ndata: ${state_json}\n\n`);
     }
   }
@@ -239,7 +282,7 @@ class Game {
   handle_player(request, response) {
     const player = this.players_by_session[request.session.id];
     const name = request.body.name;
-    const team = request.body.team;
+    const team_name = request.body.team;
     var updated = false;
     if (! player) {
       response.sendStatus(404);
@@ -257,12 +300,18 @@ class Game {
       updated = true;
     }
 
-    if (team !== null && (player.team !== team) &&
-        (team === "" || this.teams.includes(team)))
+    if (team_name !== null && (player.team.name !== team_name))
     {
-      player.team = team;
-
-      updated = true;
+      if (team_name === "") {
+        player.team = no_team;
+        updated = true;
+      } else {
+        const index = this.teams.findIndex(t => t.name === team_name);
+        if (index >= 0) {
+          player.team = this.teams[index];
+          updated = true;
+        }
+      }
     }
 
     if (updated)
@@ -272,7 +321,7 @@ class Game {
   }
 
   broadcast_move(move) {
-    this.broadcast_event("move", move);
+    this.broadcast_event("move", JSON.stringify(move));
   }
 
 }